import functools

from cpl_core.utils import String
from cpl_query.extension import List

from bot_graphql.abc.filter_abc import FilterABC


class Sort(FilterABC):
    def __init__(self):
        FilterABC.__init__(self)

        self._sort_direction = None
        self._sort_column = None

    def from_dict(self, values: dict):
        if "sortDirection" in values:
            self._sort_direction = values["sortDirection"]

        if "sortColumn" in values:
            self._sort_column = values["sortColumn"]

    @staticmethod
    def _rgetattr(obj, attr, *args):
        def _getattr(obj, attr):
            return getattr(obj, attr, *args)

        return functools.reduce(_getattr, [obj] + attr.split("."))

    def _by_column(self, x):
        atr = self._rgetattr(x, String.convert_to_snake_case(self._sort_column), None)
        if atr is None:
            return ""
        return atr

    def filter(self, query: List, *args) -> List:
        if self._sort_column is None:
            return query

        match self._sort_direction.lower() if self._sort_direction is not None else "ASC":
            case "asc":
                query = query.order_by(self._by_column)
            case "desc":
                query = query.order_by_descending(self._by_column)

        return query
